/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import com.vaadin.Application;
import com.vaadin.terminal.StreamResource;
import com.vaadin.terminal.StreamResource.StreamSource;

public class Student implements Entity, HasGetName, HasGetID {
	private int id;
	private String surname1;
	private String surname2;
	private String name;
	private String email;
	private String smsphone;
	//creado nuevo atributo de array de bytes
	private byte[] bytePhotoArray=null;
	
	
	/**
	 * Constructor para crear un Student cuando se conoce su ID, por ejemplo al
	 * cargarlo desde la BBDD. Un Student obtiene un ID cuando se guarda en la BBDD,
	 * si se crea un nuevo Student desde la aplicación (interfaz web o carga de .csv)
	 * no se conoce el ID del nuevo Student, así que no se debe utilizar este
	 * constructor.
	 * 
	 * Si el constructor recibe null en una cadena, se asignará la cadena vacía "".
	 * 
	 * @param id
	 * @param surname1
	 * @param surname2
	 * @param name
	 * @param email
	 * @param smsphone
	 */
	public Student(int id, String surname1, String surname2, String name, String email, String smsphone) {
		this.id = id;
		setSurname1(surname1);
		setSurname2(surname2);
		setName(name);
		setEmail(email);
		setSmsphone(smsphone);
	}
	
	/**
	 * Constructor para crear Student con su photoByteArray
	 * 
	 * @param id
	 * @param surname1
	 * @param surname2
	 * @param name
	 * @param email
	 * @param smsphone
	 * @param photoByteArray
	 */
	public Student(int id, String surname1, String surname2, String name, String email, String smsphone,byte[] photoByteArray) {
		this(id, surname1, surname2, name, email, smsphone);
		setBytePhotoArray(photoByteArray);
		
	}
	
	/**
	 * Constructor de los Student importados o generados por el usuario
	 * mediante la interfaz web. Al importar un Student desde un csv,
	 * se usará este constructor.
	 * 
	 * Cuando se crea un Student nuevo no se conoce su ID, por lo tanto
	 * se debe utilizar este constructor. Los objetos Student que no han
	 * sido guardados en la BBDD tienen por ID 0.
	 * 
	 * @param surname1
	 * @param surname2
	 * @param name
	 * @param email
	 * @param smsphone
	 */
	public Student(String surname1, String surname2, String name, String email, String smsphone){
		this(0, surname1, surname2, name, email, smsphone);
	}
	
	/**
	 * Constructor que permite crear un Student que sólo tenga los apellidos
	 * y nombre incializados. Permite invocar getFullName() para obtener
	 * una concatenación correcta del nombre.
	 * 
	 * @param surname1
	 * @param surname2
	 * @param name
	 */
	public Student(String surname1, String surname2, String name) {
		this(0, surname1, surname2, name, "", "");
	}
	
	/**
	 * Método estático que devuelve un array con las cabeceras del csv
	 * @return devuelve un array con las cadenas de las cabeceras que deberían tener los csv de Student
	 */
	public static String[] getBasicProperties() {
		String columns[] =  {"surname1", "surname2", "name", "email", "smsphone"};
		return columns;
	}
	
	public int getID() {
		return id;
	}
	
	public void setID(int id) {
		this.id = id;
	}
	
	public String getSurname1() {
		return surname1;
	}
	
	public void setSurname1(String surname1) {
		this.surname1 = Constants.shortString(surname1);
	}
	
	public String getSurname2() {
		return surname2;
	}
	
	public void setSurname2(String surname2) {
		this.surname2 = Constants.shortString(surname2);
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = Constants.shortString(name);
	}
	
	public String getEmail() {
		return email;
	}
	
	public void setEmail(String email) {
		this.email = Constants.shortString(email);
	}
	
	public String getSmsphone() {
		return smsphone;
	}
	
	public void setSmsphone(String smsphone) {
		this.smsphone = Constants.shortString(smsphone);
	}

	@Override
	public String toString() {
		return "Student [smsphone= " + smsphone + ", email=" + email + ", id=" + id + ", name=" + name
				+ ", surname1=" + surname1 + ", surname2=" + surname2 + "]";
	}
	
	/**
	 * Returns fullname concatenating surname1 + " " + surname2 + ", " + name.
	 * @return
	 */
	public String getFullName() {
		String fullname = surname1;
		if((!fullname.equals("")) && (!surname2.equals(""))) fullname += " ";
		fullname += surname2;
		if((!fullname.equals("")) && (!name.equals(""))) fullname += ", ";
		fullname += name;
		
		return fullname;
	}
	
	/**
	 * Se añade él mismo a la base de datos
	 * @return el id del Student insertado
	 */
	public int add() {
		return storage.addStudent(this);
	}

	/**
	 * Se elimina él mismo de la base de datos
	 */
	public void delete() {
		storage.deleteStudent(this);
	}

	/**
	 * Se modifica él mismo en la base de datos
	 * @return el código de retorno
	 */
	public int update() {
		return storage.updateStudent(this);
	}
//--->>> getters setters de array de bytes
	public byte[] getBytePhotoArray() {
		return bytePhotoArray;
	}
	public void setBytePhotoArray(byte[] bytePhotoArray) {
		this.bytePhotoArray = bytePhotoArray;
	}
	
//crea un stramresource de la array de bits
	public StreamResource getImageResource(Application app,String filename){
		
    	StreamSource streamSource = new StreamSource() {
            public InputStream getStream() {
            
            	if (bytePhotoArray!=null){
                    return new ByteArrayInputStream(bytePhotoArray);
            	}
            	else{
            		return null;
            	}
            }
        };
        return new StreamResource(streamSource, filename ,app);
	}

	
	
}
